<button class="btn btn-secondary aw-btn-sort aw-btn-sort-on aw-btn-action"><i class="fas fa-sort fa-sm"></i></button>
<button hidden="hidden" class="btn btn-secondary aw-btn-sort aw-btn-sort-asc aw-btn-action"><i class="fas fa-caret-down fa-sm"></i></button>
<button hidden="hidden" class="btn btn-secondary aw-btn-sort aw-btn-sort-desc aw-btn-action"><i class="fas fa-caret-up fa-sm"></i></button>