{% extends "../body.html" %}
{% load static %}
{% load util %}
{% block content %}
<a href="/ui/settings/permissions">
    <button class="btn btn-info aw-btn-action" title="Return">
        {% include "../button/icon/return.html" %}
    </button>
</a>

{% include "../forms/base.html" %}

<a href="/ui/settings/permissions">
    <button class="btn btn-info aw-btn-action" title="Return">
        {% include "../button/icon/return.html" %}
    </button>
</a>

{% endblock %}
