{% extends "../body.html" %}
{% load util %}
{% load static %}
{% block content %}
    <div class="aw-login">
        {% if form.errors %}
            {% for field in form %}
                {% for error in field.errors %}
                    <div class="alert alert-danger">
                        <strong>{{ error|escape }}</strong>
                    </div>
                {% endfor %}
            {% endfor %}
            {% for error in form.non_field_errors %}
                <div class="alert alert-danger">
                    <strong>{{ error|escape }}</strong>
                </div>
            {% endfor %}
        {% endif %}
        <div class="aw-success-msg aw-mark-ok">Password change successful</div>
    </div>
{% endblock %}