{% extends "../body.html" %}
{% block content %}
    <h2>Sorry, you are not allowed to access this app</h2>
    <p>To report a problem with your access please contact your system administrator</p>
    <a href="/a/login/">
        <button type="button" class="btn btn-secondary">Local Login</button>
    </a>
{% endblock %}