# Generated by Django 5.0.2 on 2024-03-03 08:45

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("aw", "0002_v0_0_13"),
    ]

    operations = [
        migrations.CreateModel(
            name="SchemaMetadata",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("created", models.DateTimeField(auto_now_add=True)),
                ("updated", models.DateTimeField(auto_now=True)),
                ("schema_version", models.CharField(max_length=50)),
                (
                    "schema_version_prev",
                    models.CharField(
                        blank=True, default=None, max_length=50, null=True
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
    ]
