{% extends "../body.html" %}
{% load static %}
{% load util %}
{% block content %}
{% if 'global' in request.GET and request.GET|get_value:"global" == 'false' %}
<h2>User Credentials</h2>
{% else %}
<h2>Global Credentials</h2>
{% endif %}

<a href="/ui/jobs/credentials">
    <button class="btn btn-info aw-btn-action" title="Return">
        {% include "../button/icon/return.html" %}
    </button>
</a>

{% include "../forms/base.html" %}

<a href="/ui/jobs/credentials">
    <button class="btn btn-info aw-btn-action" title="Return">
        {% include "../button/icon/return.html" %}
    </button>
</a>

{% endblock %}
