# Generated by Django 5.0.2 on 2024-02-28 18:37

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("aw", "0001_v0_0_12"),
    ]

    operations = [
        migrations.AddField(
            model_name="systemconfig",
            name="ara_server",
            field=models.CharField(blank=True, default=None, max_length=300, null=True),
        ),
        migrations.AddField(
            model_name="systemconfig",
            name="global_environment_vars",
            field=models.CharField(
                blank=True, default=None, max_length=1000, null=True
            ),
        ),
        migrations.AlterField(
            model_name="job",
            name="credentials_needed",
            field=models.BooleanField(
                choices=[(True, "Yes"), (False, "No")], default=False
            ),
        ),
        migrations.AlterField(
            model_name="job",
            name="inventory_file",
            field=models.CharField(blank=True, default=None, max_length=300, null=True),
        ),
        migrations.AlterField(
            model_name="jobglobalcredentials",
            name="become_user",
            field=models.CharField(
                blank=True, default="root", max_length=100, null=True
            ),
        ),
        migrations.AlterField(
            model_name="jobusercredentials",
            name="become_user",
            field=models.CharField(
                blank=True, default="root", max_length=100, null=True
            ),
        ),
    ]
