<button class="btn btn-secondary aw-btn-action aw-btn-autorefresh" title="Auto Refresh" onclick="autoReloadToggle(this);" value="OFF">
    <i class="fas fa-repeat fa-2x aw-btn-action-icon"></i>
</button>
