{% extends "../body.html" %}
{% load static %}
{% block content %}
<script src="{% static 'js/settings/api_key.js' %}"></script>
<div class="table-responsive aw-data">
    <table class="table table-striped aw-text-responsive" id="aw-api-data-table">
        <tr>
            <th>API Token</th>
            <th>Actions</th>
        </tr>
        <tr>
            <td colspan="100%">
                {% include "../button/refresh.html" %}
                <button class="btn btn-success aw-btn-action aw-api-key-add" title="Add">
                    {% include "../button/icon/add.html" %}
                </button>
            </td>
        </tr>
    </table>
    <div id="aw-api-data-tmpl-actions" hidden="hidden">
        <button class="btn btn-danger aw-btn-action aw-api-click" title="Delete" aw-api-endpoint="key" aw-api-item="${TOKEN}" aw-api-method="delete">
            {% include "../button/icon/delete.html" %}
        </button>
    </div>
</div>
{% endblock %}