<form class="aw-api-form" method="{{ form_method }}" action="/api/{{ form_api }}">
    {% csrf_token %}
    <input type="hidden" id="user" name="user" value="{{ request.user.id }}">
    {{ form }}
</form>
