AW_VALIDATIONS = {
    'file_system_browse': ['inventory_file', 'playbook_file'],
    'file_system_exists': [
        'vault_file', 'static_path', 'path_run', 'path_play', 'path_log', 'path_ansible_config',
        'path_ssh_known_hosts',
    ],
}
