<!DOCTYPE html>
{% load util %}
{% load static %}
<html lang="en">
    <head>
        {% include "./head.html" %}
        <!-- START custom header additions -->
            {% block extrahead %}
            {% endblock %}
        <!-- END custom header additions -->
    </head>
    <body>
        {% include "./nav.html" %}
        {% include "./error/js_disabled.html" %}
        <main class="aw-main">
            <div id="aw-api-error" class="aw-error-msg aw-mark-warn"></div>
            <div id="aw-ui-error" class="aw-error-msg aw-mark-warn">{{ request.GET|get_value:"error"|ignore_none }}</div>
            <div id="aw-api-result" class="aw-success-msg aw-mark-ok"></div>
            {% block content %}
            {% endblock %}
            {% if show_update_time %}
                <div id="aw-last-update"></div>
            {% endif %}
            <iframe hidden="hidden" id="aw-api-error-full" src="about:blank" width="100%" height="100%" marginheight="0" marginwidth="0" frameborder="0" scrolling="auto"></iframe>
        </main>
        <br>
        <div class="aw-footer-wrapper">
            <footer class="aw-footer">
                <div class="aw-footer-text">
                    <div>
                        © {% now 'Y' %} AnsibleGuy | Version {% get_version %}{% if user.is_authenticated %} | User: {{ request.user }}{% endif %} | <a href="https://github.com/sponsors/ansibleguy">Sponsor</a>
                    </div>
                </div>
            </footer>
        </div>
    </body>
</html>
