{% extends "../body.html" %}
{% block content %}
    <h2>You have signed out successfully.</h2>
    <p>If you want to login again or switch to another account, please do so in SSO.</p>
    <a href="/a/login/">
        <button type="button" class="btn btn-secondary">Local Login</button>
    </a>
{% endblock %}